@@ -49,7 +49,7 @@ Pl_QPDFTokenizer::finish()
 
     while (true) {
         QPDFTokenizer::Token token = this->m->tokenizer.readToken(
-            input, "offset " + QUtil::int_to_string(input->tell()), true);
+            input, "offset " + std::to_string(input->tell()), true);
         this->m->filter->handleToken(token);
         if (token.getType() == QPDFTokenizer::tt_eof) {
             break;
