@@ -128,8 +128,7 @@ QPDF::isLinearized()
             (t2.getType() == QPDFTokenizer::tt_integer) &&
             (t3 == QPDFTokenizer::Token(QPDFTokenizer::tt_word, "obj")) &&
             (t4.getType() == QPDFTokenizer::tt_dict_open)) {
-            lindict_obj =
-                QIntC::to_int(QUtil::string_to_ll(t1.getValue().c_str()));
+            lindict_obj = toI(QUtil::string_to_ll(t1.getValue().c_str()));
         }
     }
 
@@ -143,8 +142,7 @@ QPDF::isLinearized()
     }
 
     QPDFObjectHandle linkey = candidate.getKey("/Linearized");
-    if (!(linkey.isNumber() &&
-          (QIntC::to_int(floor(linkey.getNumericValue())) == 1))) {
+    if (!(linkey.isNumber() && (toI(floor(linkey.getNumericValue())) == 1))) {
         return false;
     }
 
@@ -186,24 +184,16 @@ QPDF::readLinearizationData()
 
     if (!(H.isArray() && O.isInteger() && E.isInteger() && N.isInteger() &&
           T.isInteger() && (P.isInteger() || P.isNull()))) {
-        throw QPDFExc(
-            qpdf_e_damaged_pdf,
-            this->m->file->getName(),
+        throw damagedPDF(
             "linearization dictionary",
-            this->m->file->getLastOffset(),
-            "some keys in linearization dictionary are of "
-            "the wrong type");
+            "some keys in linearization dictionary are of the wrong type");
     }
 
     // Hint table array: offset length [ offset length ]
     size_t n_H_items = toS(H.getArrayNItems());
     if (!((n_H_items == 2) || (n_H_items == 4))) {
-        throw QPDFExc(
-            qpdf_e_damaged_pdf,
-            this->m->file->getName(),
-            "linearization dictionary",
-            this->m->file->getLastOffset(),
-            "H has the wrong number of items");
+        throw damagedPDF(
+            "linearization dictionary", "H has the wrong number of items");
     }
 
     std::vector<int> H_items;
@@ -212,11 +202,8 @@ QPDF::readLinearizationData()
         if (oh.isInteger()) {
             H_items.push_back(oh.getIntValueAsInt());
         } else {
-            throw QPDFExc(
-                qpdf_e_damaged_pdf,
-                this->m->file->getName(),
+            throw damagedPDF(
                 "linearization dictionary",
-                this->m->file->getLastOffset(),
                 "some H items are of the wrong type");
         }
     }
@@ -249,12 +236,8 @@ QPDF::readLinearizationData()
     // initialized from N, to pre-allocate memory, so make sure it's
     // accurate and bail right now if it's not.
     if (N.getIntValue() != static_cast<long long>(getAllPages().size())) {
-        throw QPDFExc(
-            qpdf_e_damaged_pdf,
-            this->m->file->getName(),
-            "linearization hint table",
-            this->m->file->getLastOffset(),
-            "/N does not match number of pages");
+        throw damagedPDF(
+            "linearization hint table", "/N does not match number of pages");
     }
 
     // file_size initialized by isLinearized()
@@ -297,11 +280,8 @@ QPDF::readLinearizationData()
 
     int HSi = HS.getIntValueAsInt();
     if ((HSi < 0) || (toS(HSi) >= h_size)) {
-        throw QPDFExc(
-            qpdf_e_damaged_pdf,
-            this->m->file->getName(),
+        throw damagedPDF(
             "linearization hint table",
-            this->m->file->getLastOffset(),
             "/S (shared object) offset is out of bounds");
     }
     readHSharedObject(BitStream(h_buf + HSi, h_size - toS(HSi)));
@@ -309,11 +289,8 @@ QPDF::readLinearizationData()
     if (HO.isInteger()) {
         int HOi = HO.getIntValueAsInt();
         if ((HOi < 0) || (toS(HOi) >= h_size)) {
-            throw QPDFExc(
-                qpdf_e_damaged_pdf,
-                this->m->file->getName(),
+            throw damagedPDF(
                 "linearization hint table",
-                this->m->file->getLastOffset(),
                 "/O (outline) offset is out of bounds");
         }
         readHGeneric(
@@ -331,12 +308,8 @@ QPDF::readHintStream(Pipeline& pl, qpdf_offset_t offset, size_t length)
     qpdf_offset_t min_end_offset = oc.end_before_space;
     qpdf_offset_t max_end_offset = oc.end_after_space;
     if (!H.isStream()) {
-        throw QPDFExc(
-            qpdf_e_damaged_pdf,
-            this->m->file->getName(),
-            "linearization dictionary",
-            this->m->file->getLastOffset(),
-            "hint table is not a stream");
+        throw damagedPDF(
+            "linearization dictionary", "hint table is not a stream");
     }
 
     QPDFObjectHandle Hdict = H.getDict();
@@ -362,12 +335,8 @@ QPDF::readHintStream(Pipeline& pl, qpdf_offset_t offset, size_t length)
         *this->m->log->getError()
             << "expected = " << computed_end << "; actual = " << min_end_offset
             << ".." << max_end_offset << "\n";
-        throw QPDFExc(
-            qpdf_e_damaged_pdf,
-            this->m->file->getName(),
-            "linearization dictionary",
-            this->m->file->getLastOffset(),
-            "hint table length mismatch");
+        throw damagedPDF(
+            "linearization dictionary", "hint table length mismatch");
     }
     H.pipeStreamData(&pl, 0, qpdf_dl_specialized);
     return Hdict;
@@ -534,7 +503,7 @@ QPDF::checkLinearizationInternal()
         QPDFObjGen og(page.getObjGen());
         if (this->m->xref_table[og].getType() == 2) {
             errors.push_back(
-                "page dictionary for page " + QUtil::uint_to_string(i) +
+                "page dictionary for page " + std::to_string(i) +
                 " is compressed");
         }
     }
@@ -554,8 +523,8 @@ QPDF::checkLinearizationInternal()
         errors.push_back(
             "space before first xref item (/T) mismatch "
             "(computed = " +
-            QUtil::int_to_string(this->m->first_xref_item_offset) +
-            "; file = " + QUtil::int_to_string(this->m->file->tell()));
+            std::to_string(this->m->first_xref_item_offset) +
+            "; file = " + std::to_string(this->m->file->tell()));
     }
 
     // P: first page number -- Implementation note 124 says Acrobat
@@ -619,8 +588,8 @@ QPDF::checkLinearizationInternal()
         QTC::TC("qpdf", "QPDF warn /E mismatch");
         warnings.push_back(
             "end of first page section (/E) mismatch: /E = " +
-            QUtil::int_to_string(p.first_page_end) + "; computed = " +
-            QUtil::int_to_string(min_E) + ".." + QUtil::int_to_string(max_E));
+            std::to_string(p.first_page_end) + "; computed = " +
+            std::to_string(min_E) + ".." + std::to_string(max_E));
     }
 
     // Check hint tables
@@ -717,8 +686,8 @@ QPDF::lengthNextN(int first_object, int n, std::list<std::string>& errors)
         QPDFObjGen og(first_object + i, 0);
         if (this->m->xref_table.count(og) == 0) {
             errors.push_back(
-                "no xref table entry for " +
-                QUtil::int_to_string(first_object + i) + " 0");
+                "no xref table entry for " + std::to_string(first_object + i) +
+                " 0");
         } else {
             if (this->m->obj_cache.count(og) == 0) {
                 stopOnError("found unknown object while"
@@ -785,10 +754,9 @@ QPDF::checkHPageOffset(
         if (h_nobjects != ce.nobjects) {
             // This happens with pdlin when there are thumbnails.
             warnings.push_back(
-                "object count mismatch for page " +
-                QUtil::int_to_string(pageno) +
-                ": hint table = " + QUtil::int_to_string(h_nobjects) +
-                "; computed = " + QUtil::int_to_string(ce.nobjects));
+                "object count mismatch for page " + std::to_string(pageno) +
+                ": hint table = " + std::to_string(h_nobjects) +
+                "; computed = " + std::to_string(ce.nobjects));
         }
 
         // Use value for number of objects in hint table rather than
@@ -800,11 +768,10 @@ QPDF::checkHPageOffset(
             // This condition almost certainly indicates a bad hint
             // table or a bug in this code.
             errors.push_back(
-                "page length mismatch for page " +
-                QUtil::int_to_string(pageno) +
-                ": hint table = " + QUtil::int_to_string(h_length) +
-                "; computed length = " + QUtil::int_to_string(length) +
-                " (offset = " + QUtil::int_to_string(offset) + ")");
+                "page length mismatch for page " + std::to_string(pageno) +
+                ": hint table = " + std::to_string(h_length) +
+                "; computed length = " + std::to_string(length) +
+                " (offset = " + std::to_string(offset) + ")");
         }
 
         offset += h_length;
@@ -841,8 +808,8 @@ QPDF::checkHPageOffset(
             if (!computed_shared.count(iter)) {
                 // pdlin puts thumbnails here even though it shouldn't
                 warnings.push_back(
-                    "page " + QUtil::int_to_string(pageno) +
-                    ": shared object " + QUtil::int_to_string(iter) +
+                    "page " + std::to_string(pageno) + ": shared object " +
+                    std::to_string(iter) +
                     ": in hint table but not computed list");
             }
         }
@@ -853,8 +820,8 @@ QPDF::checkHPageOffset(
                 // built-in fonts and procsets here, at least in some
                 // cases.
                 warnings.push_back(
-                    "page " + QUtil::int_to_string(pageno) +
-                    ": shared object " + QUtil::int_to_string(iter) +
+                    "page " + std::to_string(pageno) + ": shared object " +
+                    std::to_string(iter) +
                     ": in computed list but not hint table");
             }
         }
@@ -906,8 +873,8 @@ QPDF::checkHSharedObject(
                         errors.push_back(
                             "first shared object number mismatch: "
                             "hint table = " +
-                            QUtil::int_to_string(so.first_shared_obj) +
-                            "; computed = " + QUtil::int_to_string(obj));
+                            std::to_string(so.first_shared_obj) +
+                            "; computed = " + std::to_string(obj));
                     }
                 }
 
@@ -923,8 +890,8 @@ QPDF::checkHSharedObject(
                 if (offset != h_offset) {
                     errors.push_back(
                         "first shared object offset mismatch: hint table = " +
-                        QUtil::int_to_string(h_offset) +
-                        "; computed = " + QUtil::int_to_string(offset));
+                        std::to_string(h_offset) +
+                        "; computed = " + std::to_string(offset));
                 }
             }
 
@@ -935,10 +902,10 @@ QPDF::checkHSharedObject(
             int h_length = so.min_group_length + se.delta_group_length;
             if (length != h_length) {
                 errors.push_back(
-                    "shared object " + QUtil::int_to_string(i) +
+                    "shared object " + std::to_string(i) +
                     " length mismatch: hint table = " +
-                    QUtil::int_to_string(h_length) +
-                    "; computed = " + QUtil::int_to_string(length));
+                    std::to_string(h_length) +
+                    "; computed = " + std::to_string(length));
             }
             cur_object += nobjects;
         }
@@ -985,15 +952,15 @@ QPDF::checkHOutlines(std::list<std::string>& warnings)
             if (offset != table_offset) {
                 warnings.push_back(
                     "incorrect offset in outlines table: hint table = " +
-                    QUtil::int_to_string(table_offset) +
-                    "; computed = " + QUtil::int_to_string(offset));
+                    std::to_string(table_offset) +
+                    "; computed = " + std::to_string(offset));
             }
             int table_length = this->m->outline_hints.group_length;
             if (length != table_length) {
                 warnings.push_back(
                     "incorrect length in outlines table: hint table = " +
-                    QUtil::int_to_string(table_length) +
-                    "; computed = " + QUtil::int_to_string(length));
+                    std::to_string(table_length) +
+                    "; computed = " + std::to_string(length));
             }
         } else {
             warnings.push_back("incorrect first object number in outline "
@@ -1443,7 +1410,7 @@ QPDF::calculateLinearizationData(std::map<int, int> const& object_stream_data)
             stopOnError(
                 "INTERNAL ERROR: "
                 "QPDF::calculateLinearizationData: page object for page " +
-                QUtil::uint_to_string(i) + " not in lc_other_page_private");
+                std::to_string(i) + " not in lc_other_page_private");
         }
         lc_other_page_private.erase(page_og);
         this->m->part7.push_back(pages.at(i));
@@ -1565,8 +1532,8 @@ QPDF::calculateLinearizationData(std::map<int, int> const& object_stream_data)
         stopOnError(
             "INTERNAL ERROR: QPDF::calculateLinearizationData: wrong "
             "number of objects placed (num_placed = " +
-            QUtil::uint_to_string(num_placed) +
-            "; number of objects: " + QUtil::uint_to_string(num_wanted));
+            std::to_string(num_placed) +
+            "; number of objects: " + std::to_string(num_wanted));
     }
 
     // Calculate shared object hint table information including
@@ -1828,7 +1795,7 @@ QPDF::calculateHSharedObject(
         soe.push_back(HSharedObjectEntry());
         soe.at(i).delta_group_length = length;
     }
-    if (soe.size() != QIntC::to_size(cso.nshared_total)) {
+    if (soe.size() != toS(cso.nshared_total)) {
         stopOnError("soe has wrong size after initialization");
     }
 
